<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
                xmlns:l="http://www.landxml.org/schema/LandXML-1.2"
                xmlns:h="http://xml.hexagon.com/schema/HeXML-1.8"
                exclude-result-prefixes="l h">

  <!-- **************************************************************************************** -->
  <!-- (c) 2017, Leica Geosystems AG. All rights reserved. -->
  <!-- /id          : F099EE75-4667-469b-9F9C-9EC040DB84D3 -->
  <!-- /version     : 1.0 -->
  <!-- /description : Points coordinates, code&LineID, and attributes separated by comma -->
  <!-- **************************************************************************************** -->
  
  <!-- Output ASCII -->
	<xsl:output method="text" indent="no" encoding="ISO-8859-1"/> 

	<!-- Constants variables read by Onboard exporter and attriburexporter-->
	<xsl:variable name="fileExt" select="'csv'"/>
	<xsl:variable name="fileDesc" select="'Points coordinates, code&amp;LineID, and attributes separated by comma'"/>

	<!-- Configure the default values for distances and angles values -->    
	<xsl:variable name="formatDistance" select="'0.000'"/>    
	<xsl:variable name="formatAngle" select="'0.0000'"/>
    
	<!-- Decimal format -->
	<xsl:decimal-format  NaN="--"/>
	
  <!-- Colums separators for the ASCII file -->
  <xsl:variable name="columnsSep" select="','"/>	
  
	<!-- Keys -->
	<xsl:key name="KeyPointH" match="h:Point" use="@uniqueID" />
	<xsl:key name="KeyPointL" match="l:CgPoint" use="@oID" />
    <xsl:key name="KeyLineH" match="h:PlanFeature" use="@name" />
	
	<!--
	***************************************
	Main template
	***************************************
	-->
	<xsl:template match="/">
		
		<!-- write a header for the file -->
		<xsl:text>Pt ID, E, N, Ht, Point Code, Line ID, Attrib 1, Attrib 2, Atrib 3, Attrib 4, Attrib 5, Attrib 6, Attrib 7, Attrib 8, Attrib 9, Attrib 10, Attrib 11, Attrib 12, Attrib 13, Attrib 14, Attrib 15, Attrib 16, Attrib 17, Attrib 18, Attrib 19, Attrib 20</xsl:text>		
		<xsl:call-template name="LineTerminator"/>
		
		<!-- Process the CgPoints elements sorting the points using the "timeStamp" attribute -->
		<xsl:apply-templates  select="//h:Point">
			<xsl:sort select="key('KeyPointL', @uniqueID)/@timeStamp" order="ascending"/>
		</xsl:apply-templates>
		<xsl:apply-templates  select="//l:PlanFeature">
		</xsl:apply-templates>
	</xsl:template>
               
	<!--
	***************************************
	CgPoint template
	***************************************
	-->
	<xsl:template match="h:Point">
	
		<xsl:variable name="PointH" select="."/>
	
		<xsl:variable name="CgPoint" select ="key('KeyPointL', $PointH/@uniqueID)"/>
				
    <!-- point Id -->
    <xsl:value-of select="concat($CgPoint/@oID, $columnsSep)"/>

    <!-- Local Easting -->
		<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@e, $formatDistance), $columnsSep)"/>		
		
		<!-- Local Northing -->
		<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@n, $formatDistance), $columnsSep)"/>			
		
		<!-- Ortomethic height -->
		<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@hghthO, $formatDistance), $columnsSep)"/>
		
		<!-- Point Code -->
		<xsl:value-of select="concat($PointH/h:PointCode/@code, $columnsSep, $columnsSep)"/>

    <!-- Line & String number -->
    <xsl:variable name="code">
      <!-- code -->      
      <xsl:value-of select="$PointH/h:PointCode/@code"/>

      <!-- add Line String number -->
      <xsl:call-template name="LineStringNumber">
        <xsl:with-param name="PointH" select="$PointH"/>
      </xsl:call-template>
    </xsl:variable>

    <!-- Code & attributes -->
    <xsl:if test="string-length($code) &gt; 0">
	
	  <!-- Code & lineStringNumber-->
      <!--<xsl:value-of select="concat($code, $columnsSep)"/>-->

      <!-- Point Attributes -->
      <xsl:call-template name="writeAttributesByRange">
        <xsl:with-param name="PointCodeH" select="$PointH/h:PointCode"/>
        <xsl:with-param name="attributeFst" select="1"/>
        <xsl:with-param name="attributeEnd" select="20"/>
      </xsl:call-template>
    </xsl:if>

    <!-- write the line terminator -->
		<xsl:call-template name="LineTerminator"/>	
	
	</xsl:template>
		
			
	<!--
	***************************************
	CgPlanFeatures template
	***************************************
	-->
<xsl:template match="l:PlanFeature">
	
		<xsl:variable name="PlanFeature" select="."/>
	
		<xsl:variable name="LineH" select ="key('KeyLineH', $PlanFeature/@name)"/>
				

		
		<!--<xsl:for-each select="//l:CoordGeom/*">
		<xsl:text>TestRun</xsl:text>
		</xsl:for-each>-->

		  <xsl:for-each select="$PlanFeature/l:CoordGeom/*">
			<xsl:variable name="elementName" select="string(name())"/>
			<xsl:choose>
				<xsl:when test="$elementName='Line' or @elementName='Curve'">
				<xsl:variable name="PointH" select ="key('KeyPointH', ./l:Start/@pntRef)"/>
				<xsl:if test="count (../l:Line) != substring-after(substring-after(./@oID, '_'), '_')">
				<xsl:value-of select="concat(./l:Start/@pntRef, $columnsSep)"/>
	
				<xsl:call-template name="PlanFeatureDetails">
						<xsl:with-param name="LineH" select="$LineH"/>
					    <xsl:with-param name="PointH" select="$PointH"/>
				</xsl:call-template>
				
				<!-- write the line terminator -->
				<xsl:call-template name="LineTerminator"/>	
				
			    </xsl:if>
				<xsl:if test="count (../l:Line) = substring-after(substring-after(./@oID, '_'), '_')">
				<xsl:value-of select="concat(./l:Start/@pntRef, $columnsSep)"/>
				
				<xsl:call-template name="PlanFeatureDetails">
						<xsl:with-param name="LineH" select="$LineH"/>
					    <xsl:with-param name="PointH" select="$PointH"/>
				</xsl:call-template>
				
				<!-- write the line terminator -->
				<xsl:call-template name="LineTerminator"/>	

				<!-- Update to grab the end point of the line -->
				<xsl:variable name="PointEnd" select ="key('KeyPointH', ./l:End/@pntRef)"/>
				<xsl:value-of select="concat(./l:End/@pntRef, $columnsSep)"/>
				
				<xsl:call-template name="PlanFeatureDetails">
						<xsl:with-param name="LineH" select="$LineH"/>
					    <xsl:with-param name="PointH" select="$PointEnd"/>
				</xsl:call-template>
				
				<!-- write the line terminator -->
				<xsl:call-template name="LineTerminator"/>
			    </xsl:if>
				<!--<xsl:value-of select="concat(./l:Start/@pntRef, $columnsSep)"/>
				<xsl:value-of select="concat(./@oID, $columnsSep)"/>
				<xsl:value-of select="concat(count (../l:Line), $columnsSep)"/>
				<xsl:value-of select="concat(substring-after(substring-after(./@oID, '_'), '_'), ' ')"/>-->
			</xsl:when>
			</xsl:choose>
		  </xsl:for-each>
	
		
		<!--<xsl:for-each select="//l:CoordGeom/*">
		<xsl:value-of select="concat($PlanFeature/CoordGeom/Line/Start/@pntRef, $columnsSep)"/>
		<xsl:value-of select="concat($LineH/CoordGeom/Line/@oID, $columnsSep)"/>
		<xsl:value-of select="concat($PlanFeature/@name, $columnsSep)"/>-->

	
	</xsl:template>
	
	
<xsl:template name="PlanFeatureDetails">
<xsl:param name="PointH"/>
<xsl:param name="LineH"/>
				<!-- Local Easting -->
				<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@e, $formatDistance), $columnsSep)"/>		
		
				<!-- Local Northing -->
				<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@n, $formatDistance), $columnsSep)"/>			
		
				<!-- Orthomethic height -->
				<xsl:value-of select="concat(format-number($PointH/h:Coordinates/h:Local/h:Grid/@hghthO, $formatDistance), $columnsSep)"/>

				<!-- Code Use this to check PlanFeature and H:Plan Feature codes match
				<xsl:value-of select="concat($PlanFeature/@name, $columnsSep)"/> -->
				
				<!-- Code -->
				<!--<xsl:value-of select="concat($LineH/@name, $columnsSep)"/>-->
				<xsl:value-of select="concat(substring-before($LineH/@name, '_'), $columnsSep)"/>
				<xsl:value-of select="concat(substring-after($LineH/@name, '_'), $columnsSep)"/>
				
				<!-- Point Attributes -->
				<xsl:variable name="code">
				<!-- code variable -->      
				<xsl:value-of select="$LineH/h:LineCode/@code"/>
				</xsl:variable>
				
				<xsl:if test="string-length($code) &gt; 0">
					
					<xsl:call-template name="writePlanFeatureAttributesByRange">
						<xsl:with-param name="LineCodeH" select="$LineH/h:LineCode"/>
						<xsl:with-param name="attributeFst" select="1"/>
						<xsl:with-param name="attributeEnd" select="20"/>
					</xsl:call-template>
				</xsl:if>
	</xsl:template>
	
		
	<!--
	***************************************
	Line terminator
	***************************************
	-->
	<xsl:template name="LineTerminator">        
		<xsl:text>&#13;&#10;</xsl:text>    
	</xsl:template>

  <!--
	*******************************************************************************************
	String Number Search
	*******************************************************************************************
	-->
  <xsl:template name="LineStringNumber">
    <xsl:param name="PointH"/>

    <!-- we try to find a point inside the landxml definition -->
    <xsl:variable name="stringNumberL">
      <xsl:for-each select="//l:CoordGeom/*">
        <xsl:variable name="elementName" select="string(name())"/>
        <xsl:choose>
          <xsl:when test="$elementName='Line' or @elementName='Curve'">
            <xsl:if test="./l:Start/@pntRef = $PointH/@uniqueID or 
									  ./l:End/@pntRef = $PointH/@uniqueID">
              <!-- we can have point associated to more than one line or the point could be associated several times-->
              <xsl:value-of select="concat(substring-after(./../../@name, '_'), ' ')"/>
            </xsl:if>
          </xsl:when>
        </xsl:choose>
      </xsl:for-each>
    </xsl:variable>
	
	

    <xsl:if test="string-length($stringNumberL) &gt; 0">
      <!-- we get the first occurrence of the point in the geometry -->
      <xsl:value-of select="substring-before($stringNumberL, ' ')"/>
    </xsl:if>

    <xsl:if test="string-length($stringNumberL)=0">
      <xsl:variable name="stringNumberH">
        <!-- Detect if the point is part of the spline or best fit arc -->
        <xsl:for-each select="//h:CoordGeom/*">
          <xsl:variable name="elementName" select="string(name())"/>

          <xsl:if test="$elementName='Spline' or $elementName='BestFitCurve'">
            <xsl:variable name="isPointInGeometry">
              <xsl:call-template name="IsPointIncludedInChain">
                <xsl:with-param name="pointName" select="$PointH/@uniqueID"/>
                <xsl:with-param name="chain" select="./h:Chain"/>
              </xsl:call-template>
            </xsl:variable>

            <xsl:if test="$isPointInGeometry='true'">
              <xsl:value-of select="concat(substring-after(./../../@name, '_'), ' ')"/>
            </xsl:if>
          </xsl:if>

          <xsl:if test="$elementName='Curve'">
            <xsl:if test="./h:Mid/@pntRef = $PointH/@uniqueID">
              <xsl:value-of select="concat(substring-after(./../../@name, '_'), ' ')"/>
            </xsl:if>
          </xsl:if>
        </xsl:for-each>
      </xsl:variable>

      <xsl:if test="string-length($stringNumberH) &gt; 0">
        <!-- we get the first occurrence of the point in the geometry -->
        <xsl:value-of select="substring-before($stringNumberH, ' ')"/>
      </xsl:if>

    </xsl:if>
    
  </xsl:template>

  <!--
	********************************************************************************************************
	Check if the point name is included in the chain string
	********************************************************************************************************
	-->
  <xsl:template name="IsPointIncludedInChain">
    <xsl:param name="pointName"/>
    <xsl:param name="chain"/>

    <xsl:variable name="wordLeft" select="substring-before($chain, ' ')"/>
    <xsl:variable name="wordRight" select="substring-after($chain, ' ')"/>

    <xsl:choose>
      <xsl:when test="$chain=$pointName or $wordLeft=$pointName">
        <xsl:value-of select="'true'"/>
      </xsl:when>
      <xsl:when test="string-length($wordRight) &gt; 0">
        <xsl:call-template name="IsPointIncludedInChain">
          <xsl:with-param name="pointName" select="$pointName"/>
          <xsl:with-param name="chain" select="$wordRight"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="'false'"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!--
	********************************************************************************************************
	Write the attributes from an Attribute number First & to an Atribute number End
	********************************************************************************************************
	-->
  <xsl:template name="writeAttributesByRange">
    <xsl:param name="PointCodeH"/>
    <xsl:param name="attributeFst"/>
    <xsl:param name="attributeEnd"/>

    <xsl:if test="$attributeFst &lt; $attributeEnd+1">
      <xsl:call-template name="writeAttibuteById">
        <xsl:with-param name="PointCodeH" select="$PointCodeH"/>
        <xsl:with-param name="attibuteId" select="$attributeFst"/>       
      </xsl:call-template>      
    </xsl:if>

    <xsl:if test="$attributeFst+1 &lt; $attributeEnd+1">
      <xsl:call-template name="writeAttributesByRange">
        <xsl:with-param name="PointCodeH" select="$PointCodeH"/>
        <xsl:with-param name="attributeFst" select="$attributeFst+1"/>
        <xsl:with-param name="attributeEnd" select="$attributeEnd"/>
      </xsl:call-template>
    </xsl:if>     

  </xsl:template>
  
  <!--
	********************************************************************************************************
	Write the attribute value corresponding to the Attibute number
	********************************************************************************************************
	-->
  <xsl:template name="writeAttibuteById">
    <xsl:param name="PointCodeH"/>
    <xsl:param name="attibuteId"/>

    <xsl:variable name ="attValue">
      <xsl:for-each select="$PointCodeH/h:Attribute">
        <xsl:if test="@id=$attibuteId">
          <xsl:value-of select="@value"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="string-length($attValue) &gt; 0">
        <xsl:value-of select="concat($attValue, $columnsSep)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$columnsSep"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  
  
  <!--
	********************************************************************************************************
	Write the attributes from an Attribute number First & to an Atribute number End
	********************************************************************************************************
	-->
  <xsl:template name="writePlanFeatureAttributesByRange">
    <xsl:param name="LineCodeH"/>
    <xsl:param name="attributeFst"/>
    <xsl:param name="attributeEnd"/>

    <xsl:if test="$attributeFst &lt; $attributeEnd+1">
      <xsl:call-template name="writePlanFeatureAttibuteById">
        <xsl:with-param name="LineCodeH" select="$LineCodeH"/>
        <xsl:with-param name="attibuteId" select="$attributeFst"/>       
      </xsl:call-template>      
    </xsl:if>

    <xsl:if test="$attributeFst+1 &lt; $attributeEnd+1">
      <xsl:call-template name="writePlanFeatureAttributesByRange">
        <xsl:with-param name="LineCodeH" select="$LineCodeH"/>
        <xsl:with-param name="attributeFst" select="$attributeFst+1"/>
        <xsl:with-param name="attributeEnd" select="$attributeEnd"/>
      </xsl:call-template>
    </xsl:if>     

  </xsl:template>
  
<!--
	********************************************************************************************************
	Write the plan feature attribute value corresponding to the Attibute number
	********************************************************************************************************
	-->
  <xsl:template name="writePlanFeatureAttibuteById">
    <xsl:param name="LineCodeH"/>
    <xsl:param name="attibuteId"/>

    <xsl:variable name ="attValue">
      <xsl:for-each select="$LineCodeH/h:Attribute">
        <xsl:if test="@id=$attibuteId">
          <xsl:value-of select="@value"/>
        </xsl:if>
      </xsl:for-each>
    </xsl:variable>

    <xsl:choose>
      <xsl:when test="string-length($attValue) &gt; 0">
        <xsl:value-of select="concat($attValue, $columnsSep)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$columnsSep"/>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>

  

</xsl:stylesheet>